import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:flutter_api_request/api/ApiCallBackListener.dart';
import 'package:flutter_api_request/api/HttpMethods.dart';
import 'package:flutter_api_request/api/ProgressDialog.dart';
import 'package:flutter_api_request/api/CommonFile.dart';

/* Created by Nitin Sanga  */

class ApiRequest {
  JsonDecoder jsonDecoder = new JsonDecoder();
  String url, apiAction = "", httpType = "";
  Map<String, String>? headers;
  Map<String, String>? body;
  Map<String, dynamic>? jsonResult;
  BuildContext context;
  Encoding? encoding;
  Duration connectionTimeout = Duration(minutes: 10);
  ApiCallBackListener apiCallBackListener;
  bool? showLoader = false;
  Color? progressBarColors = Colors.black;
  bool? isMultiPart = false;
  Map<String, File>? mapOfFilesAndKey;
  Function? logoutFunction;
  String? apiTimeOutMsg;
  String? errorMessage;

  ApiRequest(
      {required this.context,
        required this.apiCallBackListener,
        required this.httpType,
        required this.url,
        required this.apiAction,
        this.headers,
      this.body,
      this.encoding,
      this.isMultiPart,
      this.progressBarColors,
      this.showLoader,
      this.mapOfFilesAndKey,
      this.logoutFunction,
      this.apiTimeOutMsg,
      this.errorMessage
      }) {
    print("url=="+url.toString()+"\nshowLoader=="+showLoader.toString());
    if (context != null) {
      CommonFile.checkInternetConnectivity().then((bool isConnected) async {
        if (isConnected) {
          try {
            ProgressDialog.show(context,progressBarColors??Colors.black,showLoader??false);
            // headers = getApiHeader(accessToken);
            if (isMultiPart != null && isMultiPart==true) {
              getAPIMultiRequest(url,
                  headers: headers,
                  body: body,
                  encoding: encoding,
                  mapOfFilesAndKey: mapOfFilesAndKey);
            } else {
              getAPIRequest(url,
                  headers: headers, body: body, encoding: encoding);
            }
          } catch (onError) {
            print(onError.toString());
          }
        } else {
          CommonFile.showToastMessage("No Internet Connection.");
        }
      });
    }
  }

  getAPIRequest(String url,
      {Map<String, String>? headers, body, encoding}) async {
    print(
        "\n****************************API REQUEST************************************\n");
    print("\nApiRequest_url===" + url.toString());
    print("\nApiRequest_headers===" + headers.toString());
    print("\nApiRequest_body===" + body.toString());
    print(
        "\n****************************API REQUEST************************************\n");
    CommonFile.hideKeyBoard(context);
    Uri uri = Uri.parse(url);
    if (httpType == HttpMethods.GET) {
      return http
          .get(
            uri,
            headers: headers,
          )
          .then(httpResponse)
          .catchError(httpCatch)
          .timeout(connectionTimeout, onTimeout: () {
        apiTimeOut();
      });
    } else if (httpType == HttpMethods.POST) {
      return http
          .post(uri, headers: headers, body: body)
          .then(httpResponse)
          .catchError(httpCatch)
          .timeout(connectionTimeout, onTimeout: () {
        apiTimeOut();
      });
    } else if (httpType == HttpMethods.PUT) {
      return http
          .put(uri, headers: headers, body: body)
          .then(httpResponse)
          .catchError(httpCatch)
          .timeout(connectionTimeout, onTimeout: () {
        apiTimeOut();
      });
    } else if (httpType == HttpMethods.DELETE) {
      return http
          .delete(uri, headers: headers)
          .then(httpResponse)
          .catchError(httpCatch)
          .timeout(connectionTimeout, onTimeout: () {
        apiTimeOut();
      });
    } else if (httpType == HttpMethods.PATCH) {
      return http
          .patch(uri, headers: headers, body: body)
          .then(httpResponse)
          .catchError(httpCatch)
          .timeout(connectionTimeout, onTimeout: () {
        apiTimeOut();
      });
    }
  }

  httpCatch(onError) {
    ProgressDialog.hide();
    print("httpCatch===" + onError.toString());
    CommonFile.showToastMessage(errorMessage??"Oops! Something went wrong.");
  }

  FutureOr httpResponse(Response response) {
    try {
      ProgressDialog.hide();
      var res = response.body;
      var statusCode = response.statusCode;
      jsonResult = jsonDecoder.convert(res);
      if (statusCode == 401) {
        print(
            "\n****************************401 API RESPONSE************************************\n");

        print("\n\nApiRequest_HTTP_RESPONSE===" + jsonResult.toString());
        log("\n\nApiRequest_HTTP_BODY_RESPONSE===" + res);
        print("\n\nApiRequest_HTTP_RESPONSE_CODE===" + statusCode.toString());

        if (jsonResult != null && jsonResult!['message'] != null) {
          CommonFile.showToastMessage(jsonResult!['message'].toString());
        }
        if(logoutFunction!=null){
          logoutFunction!();
        }
        return null;
      }


      print(
          "\n****************************API RESPONSE************************************\n");

      print("\n\nApiRequest_HTTP_RESPONSE===" + jsonResult.toString());
      log("\n\nApiRequest_HTTP_BODY_RESPONSE===" + res);
      print("\n\nApiRequest_HTTP_RESPONSE_CODE===" + statusCode.toString());

      print(
          "\n****************************API RESPONSE************************************\n");

      if (jsonResult != null) {
        print("success===" + jsonResult.toString());
        return apiCallBackListener.apiCallBackListener(apiAction, jsonResult);
      } else {
        if (jsonResult != null && jsonResult!['message'] != null) {
          CommonFile.showToastMessage(jsonResult!['message'].toString());
        }
      }
    } catch (onError) {
      httpCatch(onError);
    }
  }

  apiTimeOut() {
    ProgressDialog.hide();
    print('Please try again .');
    CommonFile.showToastMessage(apiTimeOutMsg??"Connection timeout Please try again...");
  }


  Future getAPIMultiRequest(String url,
      {Map<String, String>? headers,
      body,
      encoding,
      Map<String, File>? mapOfFilesAndKey}) async {
    print(
        "\n****************************API REQUEST************************************\n");
    print("\nApiRequest_url===" + url.toString());
    print("\nApiRequest_headers===" + headers.toString());
    print("\nApiRequest_body===" + body.toString());
    print(
        "\n****************************API REQUEST************************************\n");
    var uri = Uri.parse(url);
    var request = new MultipartRequest(httpType, uri);

    List<String> keysForImage = [];
    for (int i = 0; i < mapOfFilesAndKey!.length; i++) {
      String key = mapOfFilesAndKey.keys.elementAt(i);
      keysForImage.add(key);
    }

    if(body!=null){
      for (int i = 0; i < body.length; i++) {
        String key = body.keys.elementAt(i);
        request.fields[key] = body[key];
      }
    }

    for (int i = 0; i < keysForImage.length; i++) {
      var multipartFile = await MultipartFile.fromPath(
          keysForImage[i], mapOfFilesAndKey[keysForImage[i]]!.path);
      request.files.add(multipartFile);
    }

    for(var filesdata in request.files){
      print("filesdata=="+filesdata.field.toString());
      print("filesdata=="+filesdata.filename.toString());
      print("filesdata=="+filesdata.contentType.toString());
      print("filesdata=="+filesdata.toString());
    }

    print("request.files="+request.files.toString());
    print("request.send="+request.fields.toString());
    print("request.send="+request.toString());

    request.headers.addAll(headers!);
    http.Response.fromStream(await request.send())
        .then(httpResponse)
        .catchError(httpCatch)
        .timeout(connectionTimeout, onTimeout: () {
      apiTimeOut();
    });
  }

}
